<?php
/**
 * Back Ported from 1.8.
 *
 * Elgg geo-location tagging library.
 *
 * @package Elgg.Core
 * @subpackage Location
 */

/**
 * Return entities within a given geographic area.
 *
 * Also accepts all options available to elgg_get_entities().
 *
 * @see elgg_get_entities
 *
 * @param array $options Array in format:
 *
 * 	latitude => FLOAT Latitude of the location
 *
 * 	longitude => FLOAT Longitude of the location
 *
 *  distance => FLOAT/ARR (
 *						latitude => float,
 *						longitude => float,
 *					)
 *					The distance in degrees that determines the search box. A
 *					single float will result in a square in degrees.
 *
 * radius => FLOAT A distance in degrees that determines the search radius.
 *                 This is costly and not much more practically accurate than a box.
 *
 * @warning The Earth is round.
 *
 * @see ElggEntity::setLatLong()
 *
 * @return array
 * @since 1.8.0
 */
function mfp_get_entities_from_location(array $options = array()) {

	global $CONFIG;

	$defaults = array(
		'latitude'	=>	NULL,
		'longitude'	=>	NULL,
		'distance'	=>	NULL,
		'radius'	=>	NULL
	);

	$options = array_merge($defaults, $options);

	if (!isset($options['latitude']) || !isset($options['longitude']) ||
		!(isset($options['distance']) || isset($options['radius']))) {
		return elgg_get_entities_from_relationship($options);
	}

	if (!is_array($options['distance'])) {
		$lat_distance = (float)$options['distance'];
		$long_distance = (float)$options['distance'];
	} else {
		$lat_distance = (float)$options['distance']['latitude'];
		$long_distance = (float)$options['distance']['longitude'];
	}

	$lat = (float)$options['latitude'];
	$long = (float)$options['longitude'];
	$radius = (float)$options['radius'];
	$lat_min = $lat - $lat_distance;
	$lat_max = $lat + $lat_distance;
	$long_min = $long - $long_distance;
	$long_max = $long + $long_distance;

	$where = array();
	$wheres[] = "lat_name.string='geo:lat'";
	$wheres[] = "lon_name.string='geo:long'";

	// won't work without core modifications
//	if ($radius) {
	if (false) {
		$selects[] = "((ACOS(SIN($lat * PI() / 180) * SIN(lat_value.string * PI() / 180)
			+ COS($lat * PI() / 180) * COS(lat_value.string * PI() / 180)
			* COS(($long - lon_value.string) * PI() / 180)) * 180 / PI())) as distance";

		$having = "HAVING distance < $radius";
	} else {
		$wheres[] = "lat_value.string >= $lat_min";
		$wheres[] = "lat_value.string <= $lat_max";
		$wheres[] = "lon_value.string >= $long_min";
		$wheres[] = "lon_value.string <= $long_max";
	}

	$joins = array();
	$joins[] = "JOIN {$CONFIG->dbprefix}metadata lat on e.guid=lat.entity_guid";
	$joins[] = "JOIN {$CONFIG->dbprefix}metastrings lat_name on lat.name_id=lat_name.id";
	$joins[] = "JOIN {$CONFIG->dbprefix}metastrings lat_value on lat.value_id=lat_value.id";
	$joins[] = "JOIN {$CONFIG->dbprefix}metadata lon on e.guid=lon.entity_guid";
	$joins[] = "JOIN {$CONFIG->dbprefix}metastrings lon_name on lon.name_id=lon_name.id";
	$joins[] = "JOIN {$CONFIG->dbprefix}metastrings lon_value on lon.value_id=lon_value.id";


	// merge wheres to pass to get_entities()
	if (isset($options['wheres']) && !is_array($options['wheres'])) {
		$options['wheres'] = array($options['wheres']);
	} elseif (!isset($options['wheres'])) {
		$options['wheres'] = array();
	}
	$options['wheres'] = array_merge($options['wheres'], $wheres);

	// merge joins to pass to get_entities()
	if (isset($options['joins']) && !is_array($options['joins'])) {
		$options['joins'] = array($options['joins']);
	} elseif (!isset($options['joins'])) {
		$options['joins'] = array();
	}
	$options['joins'] = array_merge($options['joins'], $joins);

	if (isset($options['selects']) && !is_array($options['selects'])) {
		$options['selects'] = array($options['selects']);
	} elseif (!isset($options['selects'])) {
		$options['selects'] = array();
	}
	$options['selects'] = array_merge($options['selects'], $selects);

	return elgg_get_entities_from_relationship($options);
}

/**
 * Returns a viewable list of entities from location
 *
 * @param array $options
 *
 * @see elgg_list_entities()
 * @see mfp_get_entities_from_location()
 *
 * @return string The viewable list of entities
 * @since 1.8.0
 */
function mfp_list_entities_from_location(array $options = array()) {
	return elgg_list_entities($options, 'mfp_get_entities_from_location');
}

/**
 * NOT IN 1.8
 *
 * Resolve a location to a lat / lon from the geocode cache table.
 *
 * @param string $location The location
 * @return array
 */
function mfp_get_lat_long_from_geocode_cache($location) {
	if (!$location) {
		return false;
	}

	$location = sanitise_string($location);
	$db_prefix = elgg_get_config('dbprefix');

	$q = "SELECT * FROM {$db_prefix}geocode_cache where location = '$location'";
	$r = get_data($q);

	if ($r) {
		$lat_lon = array(
			'latitude' => $r[0]->lat,
			'longitude' => $r[0]->long
		);

		return $lat_lon;
	}

	return false;
}

// Some distances in degrees (approximate)
// @todo huh? see warning on mfp_get_entities_from_location()
define("MILE", 0.01515);
define("KILOMETER", 0.00932);